unit IPXCOMM;

INTERFACE

uses NwMisc;

type
  MailRecord = record
                 typ,
                 size : word;
                 buffer : array[1..512] of byte;
                 node : tinternetworkaddress;
               end;

function SendMail(Mail:MailRecord):boolean;
function ReceiveMail(var Mail:MailRecord):boolean;
function CheckMail:boolean;
procedure InitIpxComm;
procedure CloseIpxComm;

function Mail2Str(mail:mailrecord):string;
function EqualNode(x,y:tnodeaddress):boolean;

const
  allnodes : tinternetworkaddress
    = (net:(0,0,0,0);
       node:($FF,$FF,$FF,$FF,$FF,$FF);
       socket:0);

var localIPXaddr :tnodeaddress;


IMPLEMENTATION

uses crt,nwipx,nwconn;

CONST IOSocket=$5678;
      EsrBufSize = 5;

Var NewStackSnd,
    NewStackRvd    :array[1..1024] of word; { !! used by ESR }
    StackBottomSnd,
    StackBottomRvd :word;                  { !! used by ESR }

    SendEcb,
    ReceiveEcb :Tecb;
    IpxHdrSnd,
    IpxHdrRvd  :TipxHeader;
    rvdbuf        :array[1..546] of byte;
    PacketSent :boolean;
    ReceivedBufLen:word;
    ReceivedMsg:array[1..EsrBufSize] of
      record
        InUse:Boolean;
        Mail :^Mailrecord;
      end;
    EsrBufInd  :Byte; { used by ESR }



{$F+}
Procedure SendESRhandler;
begin
 if SendECB.inuseflag=0 then PacketSent:=true;
end;
{$F-}

{$F+}
Procedure SendESR; assembler;
asm { ES:SI are the only valid registers when entering this procedure ! }
    mov dx, seg stackbottomsnd
    mov ds, dx

    mov dx,ss  { setup of a new local stack }
    mov bx,sp  { ss:sp copied to dx:bx}
    mov ax,ds
    mov ss,ax
    mov sp,offset stackbottomsnd
    push bx
    push dx

    CALL SendEsrHandler

    pop dx
    pop bx
    mov sp,bx { restore stack }
    mov ss,dx
end;
{$F-}

{$F+}
Procedure ListenESRhandler(Var p:Tpecb);
var i:byte;
    eq:boolean;
begin
  if ReceiveECB.inuseflag<>0 then exit;    { busy? }

  { did i send the packet ? }
  eq:=true;
  for i:=1 to 6 do
    eq:=eq and (ipxhdrrvd.source.node[i]=localipxaddr[i]);
  if not eq then begin
    { look for an empty spot in the global buffer }
    EsrBufInd:=1;
    while (EsrBufInd<=EsrBufSize) and ReceivedMsg[EsrBufInd].Inuse
     do inc(EsrBufInd);
    IF EsrBufInd<=EsrBufSize
     then begin
          { empty place found. Insert msg }
          with ReceivedMsg[EsrBufInd]
           do begin
              move(p^.fragment[2].address^,Mail^,4);
              move(p^.fragment[2].address^,Mail^,Mail^.size+4);
              Mail^.node:=tipxheader(p^.fragment[1].address^).source;
              InUse:=True;
              end;
          end
     else writeln('Buffer overrun!');
     { entire buffer is filled => discard packet }
  end;
  { Setup to listen for next incoming packet }
  IPXListenForPacket(ReceiveECB);
end;
{$F-}

{$F+}
Procedure ListenESR; assembler;
asm { ES:SI are the only valid registers when entering this procedure ! }
    mov dx, seg stackbottomrvd
    mov ds, dx

    mov dx,ss  { setup of a new local stack }
    mov bx,sp  { ss:sp copied to dx:bx}
    mov ax,ds
    mov ss,ax
    mov sp,offset stackbottomrvd
    push dx    { push old ss:sp on new stack }
    push bx

    push es    { push es:si on stack as local vars }
    push si
    mov  di,sp

    push ss    { push address of local ptr on stack }
    push di
    CALL ListenEsrHandler

    add sp,4   { skip stack ptr-copy }
    pop bx     { restore ss:sp from new stack }
    pop dx
    mov sp,bx
    mov ss,dx
end;
{$F-}

procedure InitIpxComm;
var socket:word;
    i:byte;
begin
  IF NOT IpxInitialize then begin
   writeln('Ipx needs to be installed.');
   halt(1);
  end;

  GetWorkStationNodeAddress(localIPXaddr);

  for i:=1 to Esrbufsize do
    new(ReceivedMsg[i].Mail);

  socket:=IOSocket;
  IF NOT IPXopenSocket(Socket,SHORT_LIVED_SOCKET) then begin
   writeln('IPXopenSocket returned error# ',nwIPX.result);
   halt(1);
  end;

  FillChar(rvdbuf,546,#0);

  { Setup ECB and IPX header }
  IPXsetupListenECB(Addr(ListenESR),IOsocket,@rvdbuf,546,
                    IpxHdrrvd,ReceiveEcb);

  IPXListenForPacket(ReceiveECB);
end;

procedure CloseIpxComm;
var i:byte;
begin
 IF NOT IPXcloseSocket(IOsocket) then
  writeln('IPXcloseSocket returned error# ',nwIPX.result);

  for i:=1 to Esrbufsize do
    dispose(ReceivedMsg[i].Mail);
end;


function SendMail(Mail:MailRecord):boolean;
var dest:Tinternetworkaddress;
begin
{  writeln('Mail sent: ',HexDumpStr(Mail.node.node,12),' ',
          Mail.typ,' ',Mail.size);}
  if Mail.size>512 then
    begin
      writeln('Size to big!');
      halt;
    end;
  PacketSent:=False;
  dest:=Mail.node;
  dest.socket:=iosocket;
  fillchar(dest.net,4,0);
  IPXsetupSendECB(addr(SendESR),IOsocket,dest,@mail,mail.size+4,
                  IpxHdrSnd,SendEcb);
  IPXsendPacket(SendEcb);

  REPEAT

  IpxRelinquishControl;

  UNTIL PacketSent or Keypressed;
  sendmail := packetsent;
  delay(10);
end;

function ReceiveMail(var Mail:MailRecord):boolean;
var i:byte;
begin
  i:=1;
  while (i<=EsrBufSize) and not ReceivedMsg[i].Inuse do inc(i);
  IF i<=EsrBufSize then
    begin
      Mail:=ReceivedMsg[i].Mail^;
      ReceivedMsg[i].Inuse:=False;
    end
  else ReceiveMail:=false;
end;

function CheckMail:boolean;
var i:byte;
begin
  i:=1;
  while (i<=EsrBufSize) and not ReceivedMsg[i].Inuse do inc(i);
  IF i<=EsrBufSize then CheckMail:=true
                   else CheckMail:=false;
end;

function Mail2Str(mail:mailrecord):string;
var s:string;
begin
  move(Mail.buffer,s[1],mail.size);
  s[0]:=chr(mail.size);
  Mail2Str:=s;
end;


function EqualNode(x,y:tnodeaddress):boolean;
var i:byte;
    eq:boolean;
begin
  eq:=true;
  for i:=1 to 6 do                 { did i send the packet ? }
    eq:=eq and (x[i]=y[i]);
  EqualNode:=eq;
end;

end.

  IF PacketSent
   then begin
        { ECB.InUseFlag was lowered, now determine if packet was sent: }
        CASE SendEcb.CompletionCode OF
         $00:writeln('IPX packet was sent.');
         $FC:writeln('The send of packet was canceled.');
             { impossible, as this cancelation to be done by THIS program, and it doesn't }
         $FD:writeln('Packet is malformed and was not sent.');
             { illegal param: packet length, number of fragments, fragment size. }
         $FE:writeln('Packet was undelivered. No stations listening.');
         $FF:writeln('Packet not sent due to a hardware error.');
        end;
        end;
